#!/usr/bin/perl
#
# (c) 2004 Tilman Linneweh
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the above copyright notice is
# retained.
#
# Getting started
# ###############
#
# Define a class rssitems in your stylesheet.
#
# $Id: rssfetch,v 1.34 2004/06/22 17:24:54 tilman Exp $

use vars qw/ $opt_v $opt_V $opt_h $opt_o $opt_g $opt_d $opt_c $opt_e $opt_f $opt_c $opt_F /;
use strict;
use XML::RAI;
use LWP::UserAgent;
use Getopt::Std;
use Data::Dumper;
use Config::Simple;

# version variables
my $major = 0;
my $minor = 5;
my $micro = 1;

my ($prog) = ($0 =~ /([^\/]+)$/);
sub usage {
	print STDERR <<EOF;
usage: $prog [-cdefhvV]
        -c Number	Number of topics displayed
        -d		Add date to topic;
	-e Emailaddress	Useragent From address
	-F URL		feed
	-f filename	configurationfile
        -c comment	Useragent comment
	-o filename	outputfile
	-g		debugoutput
	-h		summary of command line options
	-v		verbose output
	-V		print the version and exit
EOF
	exit 0;
}

sub version {
	print "$prog version $major.$minor.$micro\n";
	exit $major;
}

my $comment = 'http://arved.narf.at/weblog/';
my $numberofitems = 3;
my $output = "/usr/local/www/data/rssfeeds";
my $emailaddress;
my $useragent = LWP::UserAgent->new;
my $debug;
my $verbose;
my $version;
my $content;
my $configfile;
my $datedisplayed;
my %feeds;

# Parse Commandline arguments
getopts('dhvV:o:c:e:f:c:F:');
&usage if $opt_h;
&version if $opt_V;
$verbose = 1 if $opt_v;
$debug = 1 if $opt_g;
$output = $opt_o if $opt_o;
$comment = $opt_c if $opt_c;
$emailaddress = $opt_e if $opt_e;
$numberofitems = $opt_c if $opt_c;
$configfile = $opt_f if $opt_f;
$datedisplayed = $opt_d if $opt_d;
%feeds = ( '01' => $opt_F,) if $opt_F;
print "Opening Confingfile\n" if $verbose;
Config::Simple->import_from($configfile, \%feeds) if $opt_f;

$useragent->agent("rssfetch $major.$minor.$micro - $comment \n");
$useragent->from($emailaddress);
$useragent->env_proxy;
$useragent->timeout(10);

print "Starting rssfetch with verbose output\n" if $verbose;
print "Starting rssfetch with debug output\n" if $debug;
print "Outputfile: ".$output."\n" if $verbose;

open( FILE, ">$output.new" ) or die "can't open '$output.new': $!";

for ( sort keys %feeds ) {
    # retrieve feed
    print "retriving feed ".$feeds{$_}."\n" if $verbose;
    my $url = URI->new( $feeds{$_} );
    my $response = $useragent->get($url);

    #Check for empty value
    if ($response->is_success) {
	$content = $response->content;
	print "request successfull\n" if $verbose;
    } else {
        print "request was not successfull\n" if $verbose;
        print $response->content if $debug;
	next;
    }
   
    print $content if $debug;

    # XHTML defines &apos, but older brother don't support it.
    $content =~ s/&#x2019;/&#39;/g;

    #Fix broken feeds
    $content =~ s/< /&lt; /g;
    $content =~ s//&amp;sect;/g;
    print $content if $debug;
    # XXX: Fix stupid title I
    $content =~ s/baronesse de pix .*</baronesse de pix</;
 
    #parse the RSS content
    my $rai = XML::RAI->parse($content);
    print "Feed has ".@{$rai->items}." items. \n" if $verbose;
    if (@{$rai->items} == 0) {
        next;
    }
    # print the HTML channel
    print_feed($rai);
    if ($debug == 1) {
    	my $datadump = Data::Dumper->new([$rai]);
        print $datadump ->Dump."\n\n";
    }
}

close FILE;
rename "$output.new", "$output" or print "Rename failed: $!";
#  or die "can't rename '$output.new' to '$output': $!";

sub print_feed {
    my ( $rai ) = @_;
    print "Processing Feed\n" if $verbose;

    my $channeltitle = $rai->channel->title;
    my $channellink;
    if ($rai->channel->link =~ /http/g)  {
        $channellink = $rai->channel->link; 
    }
    # XXX: Hack around stupid titles II
    print "Fixing title..." if $verbose;
    $channeltitle=~ s/Wibble/lauri\'s diary/;
    $channeltitle=~ s/anorganic.org/jule - anorganic.org/;
    $channeltitle=~ s/VERFLIXT UND ZUGENEWST ! /Godany - verflixt und zugenewst! /;
    $channeltitle=~ s/Diary of a \(clever\?\) sheep./lofi - Diary of a sheep/;
    $channeltitle=~ s/ü/\&uuml\;/g;
    $channeltitle=~ s/ä/\&auml\;/g; 
    $channeltitle=~ s/Trying/friedolin - Trying/;
    $channeltitle=~ s/^Apos/friedolin - Apos/;
    $channeltitle=~ s/The Lunatic Fringe/Tim Pritlove - The Lunatic Fringe/;
    $channeltitle=~ s/a preponderance/Jonas Luster - a preponderance/;
    $channeltitle=~ s/dive into/Mark Pilgrim - dive into/;
    $channeltitle=~ s/How to learn/Francis S. - How to learn/;
    $channeltitle=~ s/aus Wiki/TomK32 - aus Wiki/;
    $channeltitle=~ s/uh/adamw - uh/;
    $channeltitle=~ s/lost and found/flata - lost and found/;
    print "done!\n" if $verbose;
    my $channeltarget= $channeltitle;
    $channeltarget=~ s/ //g;
    $channeltarget=~ s/!//g;
    $channeltarget=~ s/'//g;
    if (defined($channellink)) {
        print FILE qq(<a href="$channellink" target="$channeltarget">$channeltitle</a>);
    } else {
        print FILE qq($channeltitle);
    }
    print FILE qq(<div class="rssitems">);
    my $i = 1;
    print "Writing ".$numberofitems." items to file.." if $verbose;
    foreach my $item ( @{$rai->items} ) {
       my $topictitle;
       # check for null children 
       if (not defined($item->title)) {
           print "null title.." if $verbose;
	   $topictitle = $item->issued;
           $topictitle =~ s/T.*//g;
       } else {
           if (defined($datedisplayed)) {
               my $date = $item->issued;
               $date =~ s/T.*/ /g;
               $topictitle = $date.$item->title;
           } else { 
               $topictitle = $item->title;
           }
       }
       if (not defined($item ->link)) {
           print "null link.." if $verbose;
           next;
       }

       #debug output
       print $i.".." if $verbose;
       #if ( $debug == 1 ) {
       #    foreach ( keys %{ $item->children } ) {
       #        next unless defined($item->children($_));
       #        print $_.": ".$item->children($_)->value."\n";
       #    }
       #}

       my $topiclink = $item->link;
       # In XML & has to be encoded
       $topiclink =~ s/\&/\&amp;/g;
       $topictitle =~ s/ö/\&ouml\;/;
       $topictitle =~ s/ø/\&oslash\;/;
       $topictitle =~ s/ü/\&uuml\;/;
       $topictitle=~ s/ä/\&auml\;/g;
       $topictitle=~ s/\x9f/&szlig\;/g;
       $topictitle=~ s/\x84/\&Auml\;/g;
       print FILE qq(<a href=\"$topiclink\" target=\"$i\">$topictitle</a><br />\n);
       $i++;
       #Stop cycling over items
       if ($i == ($numberofitems + 1)) {
            last;
       }
    }
    print "done!\n" if $verbose;
    print FILE qq(</div>\n);
}
